#include "capd/capdlib.h"
#include <iostream>
using namespace std;
using namespace capd;
using namespace capd::matrixAlgorithms;

IMatrix trim(IMatrix A)
{
	int n=A.numberOfColumns()-1;
	IMatrix B(n,n);
	for(int i=0;i<n;i++)
	{
		for(int k=0;k<n;k++) B[i][k]=A[i][k];
	}
	return B;
}

bool isPositiveDefinite(IMatrix A)
{
	while(A.numberOfColumns()>0)
	{
		if(not (det(A)>0)) return 0;
		A=trim(A);
	}
	return 1;
}

IMatrix ConCondMatrix(int u,int s)
{
	IMatrix C(u+s,u+s);
	for(int i=0;i<u;i++)   C[i][i]= interval(1);
	for(int i=u;i<u+s;i++) C[i][i]=-interval(1);
	return C;
}

bool satisfiesConeCond(IMatrix DF,int u,int s,interval m)
{
	IMatrix C=ConCondMatrix(u,s);
	return isPositiveDefinite(transpose(DF)*C*DF - m*C);
}

void test()
{
	IMap F= "var:x,y,z;"
			"fun:3*x+x*y,2*y,0.5*z;";
	
	interval I = 0.1*interval(-1,1);
	IVector N({I,I,I});
	
	int u=2;
	int s=1;
	interval m=1;
	cout << satisfiesConeCond(F[N],u,s,m) << endl;
}

int main()
{
  	try
    {	
		test();
    }
  	catch(exception& e)
    {
      cout << endl << "Exception caught: " << e.what() << endl;
    }
    return 0;
}